All of lore.kernel.org
 help / color / mirror / Atom feed
From: Bart Van Assche <bvanassche@acm.org>
To: Jens Axboe <axboe@kernel.dk>
Cc: linux-block@vger.kernel.org, Christoph Hellwig <hch@lst.de>,
	Bart Van Assche <bvanassche@acm.org>,
	Hannes Reinecke <hare@suse.de>,
	Damien Le Moal <dlemoal@kernel.org>,
	Ming Lei <ming.lei@redhat.com>
Subject: [PATCH v6 10/11] block: mq-deadline: Handle requeued requests correctly
Date: Wed, 17 May 2023 10:42:28 -0700	[thread overview]
Message-ID: <20230517174230.897144-11-bvanassche@acm.org> (raw)
In-Reply-To: <20230517174230.897144-1-bvanassche@acm.org>

Start dispatching from the start of a zone instead of from the starting
position of the most recently dispatched request.

If a zoned write is requeued with an LBA that is lower than already
inserted zoned writes, make sure that it is submitted first.

Reviewed-by: Christoph Hellwig <hch@lst.de>
Reviewed-by: Hannes Reinecke <hare@suse.de>
Cc: Damien Le Moal <dlemoal@kernel.org>
Cc: Ming Lei <ming.lei@redhat.com>
Signed-off-by: Bart Van Assche <bvanassche@acm.org>
---
 block/mq-deadline.c | 34 ++++++++++++++++++++++++++++++++--
 1 file changed, 32 insertions(+), 2 deletions(-)

diff --git a/block/mq-deadline.c b/block/mq-deadline.c
index 91b689261d30..e90879869c90 100644
--- a/block/mq-deadline.c
+++ b/block/mq-deadline.c
@@ -156,13 +156,28 @@ deadline_latter_request(struct request *rq)
 	return NULL;
 }
 
 /* Return the first request for which blk_rq_pos() >= pos. */
 static inline struct request *deadline_from_pos(struct dd_per_prio *per_prio,
 				enum dd_data_dir data_dir, sector_t pos)
 {
 	struct rb_node *node = per_prio->sort_list[data_dir].rb_node;
 	struct request *rq, *res = NULL;
 
+	if (!node)
+		return NULL;
+
+	rq = rb_entry_rq(node);
+	/*
+	 * A zoned write may have been requeued with a starting position that
+	 * is below that of the most recently dispatched request. Hence, for
+	 * zoned writes, start searching from the start of a zone.
+	 */
+	if (blk_rq_is_seq_zoned_write(rq))
+		pos -= round_down(pos, rq->q->limits.chunk_sectors);
+
 	while (node) {
 		rq = rb_entry_rq(node);
 		if (blk_rq_pos(rq) >= pos) {
@@ -806,6 +821,8 @@ static void dd_insert_request(struct blk_mq_hw_ctx *hctx, struct request *rq,
 		list_add(&rq->queuelist, &per_prio->dispatch);
 		rq->fifo_time = jiffies;
 	} else {
+		struct list_head *insert_before;
+
 		deadline_add_rq_rb(per_prio, rq);
 
 		if (rq_mergeable(rq)) {
@@ -818,7 +835,20 @@ static void dd_insert_request(struct blk_mq_hw_ctx *hctx, struct request *rq,
 		 * set expire time and add to fifo list
 		 */
 		rq->fifo_time = jiffies + dd->fifo_expire[data_dir];
-		list_add_tail(&rq->queuelist, &per_prio->fifo_list[data_dir]);
+		insert_before = &per_prio->fifo_list[data_dir];
+#ifdef CONFIG_BLK_DEV_ZONED
+		/*
+		 * Insert zoned writes such that requests are sorted by
+		 * position per zone.
+		 */
+		if (blk_rq_is_seq_zoned_write(rq)) {
+			struct request *rq2 = deadline_latter_request(rq);
+
+			if (rq2 && blk_rq_zone_no(rq2) == blk_rq_zone_no(rq))
+				insert_before = &rq2->queuelist;
+		}
+#endif
+		list_add_tail(&rq->queuelist, insert_before);
 	}
 }
 
@@ -156,13 +156,28 @@ deadline_latter_request(struct request *rq)
 }
 
-/* Return the first request for which blk_rq_pos() >= pos. */
+/*
+ * Return the first request for which blk_rq_pos() >= @pos. For zoned devices,
+ * return the first request after the start of the zone containing @pos.
+ */
 static inline struct request *deadline_from_pos(struct dd_per_prio *per_prio,
 				enum dd_data_dir data_dir, sector_t pos)

  parent reply	other threads:[~2023-05-17 17:45 UTC|newest]

Thread overview: 20+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2023-05-17 17:42 [PATCH v6 00/11] mq-deadline: Improve support for zoned block devices Bart Van Assche
2023-05-17 17:42 ` [PATCH v6 01/11] block: mq-deadline: Add a word in a source code comment Bart Van Assche
2023-05-18 23:11   ` Damien Le Moal
2023-05-18 23:20     ` Bart Van Assche
2023-05-18 23:39       ` Damien Le Moal
2023-05-17 17:42 ` [PATCH v6 02/11] block: Simplify blk_req_needs_zone_write_lock() Bart Van Assche
2023-05-17 17:42 ` [PATCH v6 03/11] block: Fix the type of the second bdev_op_is_zoned_write() argument Bart Van Assche
2023-05-17 17:42 ` [PATCH v6 04/11] block: Introduce op_needs_zoned_write_locking() Bart Van Assche
2023-05-18 23:12   ` Damien Le Moal
2023-05-17 17:42 ` [PATCH v6 05/11] block: Introduce blk_rq_is_seq_zoned_write() Bart Van Assche
2023-05-18 23:13   ` Damien Le Moal
2023-05-17 17:42 ` [PATCH v6 06/11] block: mq-deadline: Clean up deadline_check_fifo() Bart Van Assche
2023-05-18 23:14   ` Damien Le Moal
2023-05-17 17:42 ` [PATCH v6 07/11] block: mq-deadline: Simplify deadline_skip_seq_writes() Bart Van Assche
2023-05-17 17:42 ` [PATCH v6 08/11] block: mq-deadline: Reduce lock contention Bart Van Assche
2023-05-17 17:42 ` [PATCH v6 09/11] block: mq-deadline: Track the dispatch position Bart Van Assche
2023-05-17 17:42 ` Bart Van Assche [this message]
2023-05-18 23:15   ` [PATCH v6 10/11] block: mq-deadline: Handle requeued requests correctly Damien Le Moal
2023-05-17 17:42 ` [PATCH v6 11/11] block: mq-deadline: Fix handling of at-head zoned writes Bart Van Assche
2023-05-19  1:50 ` [PATCH v6 00/11] mq-deadline: Improve support for zoned block devices Jens Axboe

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20230517174230.897144-11-bvanassche@acm.org \
    --to=bvanassche@acm.org \
    --cc=axboe@kernel.dk \
    --cc=dlemoal@kernel.org \
    --cc=hare@suse.de \
    --cc=hch@lst.de \
    --cc=linux-block@vger.kernel.org \
    --cc=ming.lei@redhat.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.